;==============================================================================
;
;	proba02.ASM		HARDWARE HASZNLATTAL (LED-ek s GOMB-ok)
;
;	2. rai AVR ASM 2. feladatrsz, progik (ismtls s j anyag)
;
;	F7:      Assemble,		
;	Ctrl+F7: Assemble and Run
;	F11:	 Step
;
;------------------------------------------------------------------------------
;
;	DDRx  (8bit)	Data Direction (O:be, 1:ki) ( PORT irny bellits)
;	PORTx (8bit)	1 PORT kimeneti rtknek a belltsa (0: 0V, 1: 5V)
;	PINx  (1bit)	1 lb  kimeneti rtknek a belltsa (0: 0V, 1: 5V)
;
;==============================================================================

;==============================================================================
; 1. FELADAT: GOMB-ok s LED-ek kezelse mlesztve
;
;	      - ha megnyomunk egy GOMB-ot, akkor vilgt a megfelel LED
;
;	FONTOS: Els lpsben az egyes regiszterek, flag-ek rtkeit, 
;               S a nyomgomb megnyomst (PING rtkt), a LED vilgtst 
;               szimultorban kvessk nyomon.
;
;		A program megrtse utn tltsk fel az eszkzre, s prbljuk ki
;------------------------------------------------------------------------------

.include "m128def.inc"
.def tmp =r16		; segdvltoz elnevezs
.def gomb=r17		; GOMB llapott lekrdez regiszter
.def led =r18		; LED meghajt regiszter

.org 0x0
	rjmp start
.org 0x100
start:
				; LED-eket vezrl PORT-ok belltsa PORTB, PORTD
	ldi tmp,   0b11110000	; fels 4 bit 1-es, als 4 bit 0-s
	out DDRB,  tmp		; fels 4 bit KImenet, als 4 bit BEmenet
	out DDRD,  tmp		; fels 4 bit KImenet, als 4 bit BEmenet

				; GOMB-okat vezrl PORTG bellitsa BEmenetre
	ldi	tmp,   0b00000000	; temp vltoz csupa 0, BEmenetre lltva
	sts	DDRG,  tmp		; PORTG bemenet

CIKLUS:
	lds  gomb,  PING		; gomb nev reg-be olvassuk a gombok llapotot
	andi gomb,	0b00011111	; ki kell maszkolni, hogy a tbbi ne zavarjon
	mov  led, gomb			; led nev reg-be betltjk a gomb rtkt

	out  PORTD, led	; led rtkt kitesszk a PORTD-re
	swap led	; felcserljk led nev regiszter als s fels 4 bitjt
	out  PORTB, led	; led rtkt kitesszk a PORTB-re
	swap led	; swap vissza (BAB)

jmp CIKLUS

vege:			; BAB
	jmp vege

;==============================================================================
; 2. FELADAT: GOMB-ok s LED-ek kezelse MAKR-val
;
;	      - ugyanaz, mint az elbbi progi, csak MAKR-val megoldva
;	      -	ha megnyomunk egy GOMB-ot, akkor vilgt a megfelel LED
;
;	FONTOS: Els lpsben az egyes regiszterek, flag-ek rtkeit, 
;               S a nyomgomb megnyomst (PING rtkt), a LED vilgtst 
;               szimultorban kvessk nyomon.
;
;		A program megrtse utn tltsk fel az eszkzre, s prbljuk ki
;------------------------------------------------------------------------------

.include "m128def.inc"
.def tmp =r16		; segdvltoz elnevezs
.def gomb=r17		; GOMB llapott lekrdez regiszter
.def led =r18		; LED meghajt regiszter

.macro PORTOK_beallitasa_M	; ---- MAKRO a hardware LED s GOMB PORT-ok belltsra
				; LED-eket vezrl PORT-ok belltsa PORTB, PORTD
	ldi tmp,   0b11110000	; fels 4 bit 1-es, als 4 bit 0-s
	out DDRB,  tmp		; fels 4 bit KImenet, als 4 bit BEmenet
	out DDRD,  tmp		; fels 4 bit KImenet, als 4 bit BEmenet

				; GOMB-okat vezrl PORTG bellitsa BEmenetre
	ldi	tmp,   0b00000000	; temp vltoz csupa 0, BEmenetre lltva
	sts	DDRG,  tmp		; PORTG bemenet

.endmacro			; ---- MAKRO VGE ---- ---- ---- ---- ---- ---- 

.org 0x0			; ---- PROGI kezdete ---- ----
	rjmp start
.org 0x100
start:

	PORTOK_beallitasa_M	; ---- MAKRO meghvsa ---- ---- ---- ---- ----

CIKLUS:
	lds  gomb,  PING		; gomb nev reg-be olvassuk a gombok llapotot
	andi gomb,	0b00011111	; ki kell maszkolni, hogy a tbbi ne zavarjon
	mov  led, gomb			; led nev reg-be betltjk a gomb rtkt

	out  PORTD, led	; led rtkt kitesszk a PORTD-re
	swap led	; felcserljk led nev regiszter als s fels 4 bitjt
	out  PORTB, led	; led rtkt kitesszk a PORTB-re
	swap led	; swap vissza (BAB)

jmp CIKLUS

vege:			; BAB
	jmp vege

;==============================================================================
; 3. FELADAT: LED-es futfny s DELAY bemutatsa (MAKRO is van a delay-nl)
;
;		DELAY-t (ksleltetst) szmllk prgetsvel csinljuk 
;------------------------------------------------------------------------------

.include "m128def.inc"

.def tmp=r16
.def LED=r17

;--------------------------------------------------------------
.macro DELAY_macro				

	ldi R26, 0b11111111	; regiszterekbe beadok valamilyen szmot
	ldi R25, 0b11111111
	ldi R24, 0b00001111

	delay_loop:
		dec  R26	; lefele szmol, ha 0 lesz, akkor Z flag=1
		brne delay_loop ; Z flag-et nzi, ha Z flag=0, akkor ugrik, 
				; mert akkor a r26 szm nem 0
		dec  R25
		brne delay_loop

		dec  R24
		brne delay_loop
.endmacro
;--------------------------------------------------------------

.org 0x0
	jmp start

.org 0x100
start:

	ldi tmp, 0b_1111_0000	; segdvltoz a PORTirny belltshoz
	out DDRB, tmp		; PORTB fels 4 bitje 1 (KI-m), als 4 bit 0 (BE-m)
	out DDRD, tmp		; PORTD fels 4 bitje 1 (KI-m), als 4 bit 0 (BE-m)

	ldi tmp, 0b_0000_0001	; 1-et tltnk be a TMP nev regiszterbe 
				; (alapbl egy LED vilgt) {Mi van, ha 3-at tltnk be?}
	clc			; CLEAR Carry (Carry trls)

loop:
	  out  PORTB, tmp 	; TMP-et betesszk PORTB-be ( PORTB fels 4 bitje a lnyeg, LED!)
	  swap tmp		; fels 4 s als 4 bit csere
	  out  PORTD, tmp 	; TMP eredeti als 4 bitjt betesszk PORTD fels 4 bitjre 
				; (PORTD fels 4 bitje a lnyeg, LED!)
	  swap tmp		; visszalltjuk az eredeti szmot, hogy utna tudjuk lptetni

	  rol  tmp		; TMP, ROTATE LEFT th. CARRY

	  DELAY_macro		; DELAY (lassts) meghvsa

	jmp loop		; vgtelensgig prg

vege:				; BAB
	jmp vege
