;==============================================================================
;
;	MAKR, STACK, Szubrutin, IT hasznlat
;
;	3. rai AVR ASM 1. feladatrsz, progik 
;
;		- Makrk hasznlata
;
;		- STACK bemutatsa
;
;		- Szubrutin hasznlata
;
;		- IT hasznlat
;
;	F7:      Assemble,		
;	Ctrl+F7: Assemble and Run
;	F11:	 Step
;
;------------------------------------------------------------------------------
;
;	DDRx  (8bit)	Data Direction (O:be, 1:ki) ( PORT irny bellits)
;	PORTx (8bit)	1 PORT kimeneti rtknek a belltsa (0: 0V, 1: 5V)
;	PINx  (1bit)	1 lb  kimeneti rtknek a belltsa (0: 0V, 1: 5V)
;			FONTOS: csak egsz PORT-ot lehet beolvasni (egyszerre 8 bitet)
;
;==============================================================================

;==============================================================================
; 1. FELADAT: STACK kezels bemutatsa, SUBRUTIN-hoz kell mr
;------------------------------------------------------------------------------
;
;
;	View -> Memory ablak megnyitsa -> ADATMEMRIA kivlasztsa
;
;	FONTOS: - innentl kezdve, ha kell, ha nem, STACK-et mindig inicializljuk!
;		- a STACK a SZUBRUTIN-hoz s a MEGSZAKTS-hoz (IT kezelsnl) ktelez!
;
;	FONTOS: - a SP rtkt s az adatmemria vgt nyomon kell kvetni
;============================================================================= 

.include "m128def.inc"
.def a=r16
.def b=r17
.def c=r18

.org 0x0
	rjmp start

.org 0x100
start:

; --- --- --- --- STACK belltsa a vgre --- --- --- --- --- --- --- ---
	ldi	r16, HIGH(RAMEND)	; stack bellts, a memria vgre teszi a stack pointer-t
	out	SPH, r16		; elsknt a STACK fels BYTE-jt lltja
	ldi	r16, LOW(RAMEND)	; utna    a STACK als  BYTE-jt lltja
	out 	SPL, r16		; de a BYTE-ok lltsa lehetne fordtva is
; --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- ---
	ldi	a, 11		; "a" nev regiszterbe (r16) betltnk 11-et
	ldi	b, 12		; "b" nev regiszterbe (r17) betltnk 12-t
	ldi	c, 13		; "c" nev regiszterbe (r18) betltnk 13-at

; --- --- --- --- STACK kezelse --- (LIFO, Last IN => First OUT) --- --- --- 
	nop
	push	a	; "a" rtkt betesszk a stack-be
	push	b	; "b" rtkt betesszk a stack-be
	push	c	; "c" rtkt betesszk a stack-be

	nop
	clr	a	; "a" regiszter rtkt trlljk
	clr	b	; "b" regiszter rtkt trlljk
	clr	c	; "c" regiszter rtkt trlljk

	nop
	pop	c	; rtket kivesszk a STACK-bl s betesszk "c"-be
	pop	b	; rtket kivesszk a STACK-bl s betesszk "b"-ba
	pop	a	; rtket kivesszk a STACK-bl s betesszk "a"-ba

vege:			; itt prg a program a vgtelensgig
	jmp vege

;==================================================================================
; 2. FELADAT: sszeads megoldsa, kzvetlen regiszterrel (A), MAKRO-val (B), SZUBRUTIN-nal (C)
;----------------------------------------------------------------------------------
;
;		- knny plda szimultorban bemutathat
;				
;		- MAKR s SZUBRUTIN mkds bemutatsa (2 szm sszeadsa)
;
;			- sszeads kzvetlen regiszterrel   (A)
;			- sszeads MAKR alkalmazsval     (B)
;			- sszeads SZUBRUTIN alkalmazsval (C)
;
;		- Labor gyakorlaton rszletekben is mutathat: 
;			- elszr az sszeads kzvetlen regiszterrel,
;			- utna az sszeads MAKR alkalmazsval,
;			- vgl az sszeads SZUBRUTIN alkalmazsval
;================================================================================== 

.include "m128def.inc"
.def a=r16
.def b=r17
.def SAVE_reg=r24

.macro MAKRO_osszead
	add a, b; "a" s "b" nev regiszterek sszeadsa
.endmacro

.org 0x0
	rjmp start

.org 0x100
start:
		; -------------------------------------------
	ldi	r16, HIGH(RAMEND)	; stack bellts
	out	SPH, r16		; a memria vgre teszi
	ldi	r16, LOW(RAMEND)	; a stack pointer-t
	out 	SPL, r16
		; -------------------------------------------			
	ldi	a, 2		; "a" nev regiszterbe (r16) betltnk 2-t
	ldi	b, 4		; "b" nev regiszterbe (r17) betltnk 4-et
	add	a, b		; sszeads (A)
	nop
		; --- "a" s "b" regiszterek eredeti rtkeinek a visszalltsa, 
		; --- hogy ugyanazt tudjam mutatni MAKRO-val
	ldi	a, 2		; "a" nev regiszterbe (r16) betltnk 2-t
	ldi	b, 4		; "b" nev regiszterbe (r17) betltnk 4-et

	MAKRO_osszead		; MAKRO-s sszeads
	nop
		; --- "a" s "b" regiszterek eredeti rtkeinek a visszalltsa, 
		; --- hogy ugyanazt tudjam mutatni SZUBRUTIN-nal
	ldi	a, 2		; "a" nev regiszterbe (r16) betltnk 2-t
	ldi	b, 4		; "b" nev regiszterbe (r17) betltnk 4-et

	call SZUBRUTIN_osszead	; Szubrutin meghvsa, sszeads ott
	nop

vege:			; itt prg a program a vgtelensgig
	jmp vege

; ------- Szubrutinok IDE -----------------------------

SZUBRUTIN_osszead:		; sszeads szubrutin lersa

	add a, b		; "a" s "b" nev regiszterek sszeadsa
	ret

;==============================================================================
; 3. FELADAT: Futfny s DELAY mlesztve
;------------------------------------------------------------------------------

.include "m128def.inc"

.def tmp=r16

.org 0x0				; kezdcm, itt indul a progi
	jmp start			; jmp utasts 0x0 cmen van

.org 0x100				; cm
start:					; start cmke 0x100 cmen van

		; --- --- --- --- STACK belltsa --- --- --- --- --- --- --- ---
	ldi	tmp, HIGH(RAMEND)	; stack bellts
	out	SPH, tmp		; a memria vgre teszi
	ldi	tmp, LOW(RAMEND)	; a stack pointer-t
	out 	SPL, tmp
		; --- --- --- --- PORT-ok kezelse --- --- --- --- --- --- --- ---
	ldi tmp, 0b11110000		; segdvltoz a PORTirny belltshoz
	out DDRB, tmp			; PORTB fels 4 bitje 1 (KIm), als 4 bit 0 (BEm)
	out DDRD, tmp			; PORTD fels 4 bitje 1 (KIm), als 4 bit 0 (BEm)

	ldi tmp, 0b_0000_0001		; 1-et tltnk be a TMP nev regiszterbe 
					; (alapbl egy LED vilgt) {Mi van, ha 3-at tltnk be?}
	clc				; CLEAR Carry (Carry trls)

loop:
	out  PORTD, tmp 	; TMP-et betesszk PORTD-be ( PORTD fels 4 bitje a lnyeg, LED!)
	swap tmp		; fels 4 bit s als 4 bit csere
	out  PORTB, tmp 	; TMP eredeti als 4 bitjt betesszk PORTB fels 4 bitjre 
				; (PORTB fels 4 bitje a lnyeg, LED!)
	swap tmp		; visszalltjuk a szmot, hogy utna tudjuk lptetni		

	rol	 tmp		; TMP, ROTATE LEFT th. CARRY

	;--------------------------------------------------------------
		ldi R26, 0b_1111_1111	; regiszterekbe beadunk egy rtket
		ldi R25, 0b_1111_1111
		ldi R24, 0b_0000_1111
	delay_loop:
		dec  R26		; lefele szmol, ha 0 lesz, akkor Z flag=1
		brne delay_loop 	; Z flag-et nzi, ha Z flag=0, akkor ugrik, mert akkor a r26 rtke nem 0

		dec  R25
		brne delay_loop
	
		dec  R24
		brne delay_loop
	;--------------------------------------------------------------

	jmp loop

vege:			; BAB!
	jmp vege

;==============================================================================
; 4. FELADAT: Futfny s DELAY , de a DELAY MAKRO-ban van
;------------------------------------------------------------------------------

.include "m128def.inc"

.def tmp=r16

.macro delay
	ldi R26, 0b11111111	; regiszterekbe beadunk egy rtket
	ldi R25, 0b11111111
	ldi R24, 0b00001111
	delay_loop:
		dec  R26		; lefele szmol, ha 0 lesz, akkor Z flag=1
		brne delay_loop 	; Z flag-et nzi, ha Z flag=0, akkor ugrik, mert akkor a r26 rtke nem 0

		dec  R25
		brne delay_loop
	
		dec  R24
		brne delay_loop
.endmacro

.org 0x0				; kezdcm, itt indul a progi
	jmp start			; jmp utasts 0x0 cmen van

.org 0x100				; cm
start:					; start cmke 0x100 cmen van

		; --- --- --- --- STACK belltsa a vgre --- --- --- --- --- --- --- ---
	ldi	tmp, HIGH(RAMEND)	; stack bellts
	out	SPH, tmp		; a memria vgre teszi
	ldi	tmp, LOW(RAMEND)	; a stack pointer-t
	out 	SPL, tmp

		; --- --- --- --- PORT-ok kezelse --- --- --- --- --- --- --- ---
	ldi tmp, 0b11110000		; segdvltoz a PORTirny belltshoz
	out DDRB, tmp			; PORTB fels 4 bitje 1 (KIm), als 4 bit 0 (BEm)
	out DDRD, tmp			; PORTD fels 4 bitje 1 (KIm), als 4 bit 0 (BEm)

	ldi tmp, 0b_0000_0001		; 1-et tltnk be a TMP nev regiszterbe 
					; (alapbl egy LED vilgt) {Mi van, ha 3-at tltnk be?}

loop:
	out  PORTD, tmp 	; TMP-et betesszk PORTD-be ( PORTD fels 4 bitje a lnyeg, LED!)
	swap tmp		; fels 4 bit s als 4 bit csere
	out  PORTB, tmp 	; TMP eredeti als 4 bitjt betesszk PORTB fels 4 bitjre 
				; (PORTB fels 4 bitje a lnyeg, LED!)
	swap tmp		; visszalltjuk a szmot, hogy utna tudjuk lptetni		

	rol  tmp		; TMP, ROTATE LEFT th. CARRY
	
	delay			; 'delay' nev MAKRO meghvsa			

	jmp loop

vege:			; BAB!
	jmp vege

;==============================================================================
; 5. FELADAT: Futfny s DELAY , de a DELAY SZUBRUTIN-ban van
;------------------------------------------------------------------------------

.include "m128def.inc"

.def tmp=r16

.org 0x0				; kezdcm, itt indul a progi
	jmp start			; jmp utasts 0x0 cmen van

.org 0x100				; cm
start:					; start cmke 0x100 cmen van

; --- --- --- --- STACK belltsa a vgre --- --- --- --- --- --- --- ---
	ldi	tmp, HIGH(RAMEND)	; stack bellts
	out	SPH, tmp		; a memria vgre teszi
	ldi	tmp, LOW(RAMEND)	; a stack pointer-t
	out 	SPL, tmp

; --- --- --- --- PORT-ok belltsa --- --- --- --- --- --- --- --- 
	ldi tmp, 0b11110000		; segdvltoz a PORTirny belltshoz
	out DDRB, tmp			; PORTB fels 4 bitje 1 (KIm), als 4 bit 0 (BEm)
	out DDRD, tmp			; PORTD fels 4 bitje 1 (KIm), als 4 bit 0 (BEm)

	ldi tmp, 0b_0000_0001		; 1-et tltnk be a TMP nev regiszterbe 
					; (alapbl egy LED vilgt) {Mi van, ha 3-at tltnk be?}

loop:
	out  PORTD, tmp 	; TMP-et betesszk PORTD-be ( PORTD fels 4 bitje a lnyeg, LED!)
	swap tmp		; fels 4 bit s als 4 bit csere
	out  PORTB, tmp 	; TMP eredeti als 4 bitjt betesszk PORTB fels 4 bitjre 
				; (PORTB fels 4 bitje a lnyeg, LED!)
	swap tmp		; visszalltjuk a szmot, hogy utna tudjuk lptetni		

	rol  tmp		; TMP, ROTATE LEFT th. CARRY
	
	call delay		; 'delay' nev SZUBRUTIN meghvsa			
	jmp loop

vege:			; BAB!
	jmp vege

;--- SZUBRUTIN-ok -----------------------------------------------------

delay:
	ldi R26, 0b_1111_1111	; regiszterekbe beadok valamilyen szmot
	ldi R25, 0b_1111_1111
	ldi R24, 0b_0000_1111
	delay_loop:
		dec  R26		; lefele szmol, ha 0 lesz, akkor Z flag=1
		brne delay_loop 	; Z flag-et nzi, ha Z flag=0, akkor ugrik, mert akkor a r26 rtke nem 0

		dec  R25
		brne delay_loop
	
		dec  R24
		brne delay_loop
ret

;==============================================================================
; 6. FELADAT: Futfny megvalstsa IT0 MEGSZAKTS-sal
;------------------------------------------------------------------------------
;				
;		Megszakts, Interrupt, IT0 mkds bemutatsa
;
;		LED futfny megvalstsa IT0-s INTERRUPT-tal
;
;============================================================================== 

.include "m128def.inc"
.def tmp =r16		; segdvltoz
.def led =r18		; LED meghajt regiszter
.def tmp_s=r19		; STACK mentsre szolgl segdvltoz (opcionlisan hasznlhat)
;------------------------------------------------------------

.org 0x0		; Ezen a cmen indul a PROGI
	rjmp start

.org 0x20
	rjmp IT0_OVF	; IT0 Overflow megszakts bejvetelnek jelzse az 0x20-as cmen van, 
			; ha a IT0 8 bites szmllja tlcsordul, akkor ad egy IT-t 0x20 cmre.
.org 0x100
start:		
			; --- STACK inicializlsa ------------------
	ldi	tmp, HIGH(RAMEND)	; stack bellts
	out	SPH, tmp		; a memria vgre teszi
	ldi	tmp, LOW(RAMEND)	; a stack pointer-t
	out 	SPL, tmp
	clr 	tmp
			; --- LED-eket vezrl PORT-ok belltsa PORTB, PORTD ---
	ldi tmp,   0b11110000	; PORT-ok fels 4 bitje kimenetre lltani	
	out DDRB,  tmp
	out DDRD,  tmp
	
	ldi led, 0b0000_0001	; LED vltozba a kezdeti szm
	clc
			; --- IT0 kezdeti belltsa ----------------
	ldi tmp, 	0b0000_0111	; segdvltoz a belltshoz
	out TCCR0,	tmp		; 105. oldal DSh.-bl, CLK/1024 elosztst lltok be
	ldi tmp, 	0b0000_0001
	out TIMSK,	tmp		; IT0-s INTERRUPT engedlyezst lltok be,
					; ha az IT0 8 bites szmllja tlcsordul, 
					; akkor ad egy INTERRUPT-ot

	sei				; proci engedi befogadni a bejv IT-t
			; --- IT0 kezdeti bellts VGE ----------------


CIKLUS:			; ITT PRG A PROGI, ltszlag nem csinl semmit
	sleep		; kisfogyaszts llapot
	jmp CIKLUS


; --- Megszakts IT0 ------------------------ 
			; LTHAT, hogy az IT0-t nem a Fprogibl hvjuk,
			; hanem egy kls HW kezdemnyezi, s utna,
			; ha a PROCI elfogadja a megszaktst (SEI), 
			; akkor lefut az IT0-hoz tartoz IT rutin.

IT0_OVF:
			; CSAK minden 16. IT megjvetelekor lp a LED 	(LASSTS)
	; dec tmp	; 1-el cskkentjk a tmp rtkt		(LASSTSHOZ KIVENNI a kommentet)
	; brne cimke1	; ugrik a cimke1-re, ha tmp nem "0" 		(LASSTSHOZ KIVENNI a kommentet)  

	; ldi tmp, 16	; ha tmp=0, akkor belemegy s egybl kezdeti rtket adni neki, itt tmp=16 (LASSTSHOZ KIVENNI a kommentet)

	out  PORTD, led	; kitenni a LED vltoz fels 4 bitjt, a PORTD fels 4 bitjre (LED meghajts)
	swap led	; fels 4 s als 4 bit csere
	out  PORTB, led ; kitenni a LED vltoz eredeti als 4 bitjt, a PORTB fels 4 bitjre (LED meghajts)
	swap led	; visszalltjuk a szmot, hogy utna tudjuk lptetni

	rol	 led	; LED, ROTATE LEFT th. CARRY
cimke1:

reti

;==============================================================================
; 7. FELADAT: 3 szn LED szneinek bekapcsolsa, majd kikapcsolsa
;
;	FONTOS: a feladat vgeztvel a LED-et kapcsoljuk ki (ne menjen egsz jjel)
;
;------------------------------------------------------------------------------

.include "m128def.inc"
.def tmp=r16			; segdvltoz

.org 0x0			; kezdcm
	rjmp start

.org 0x100			; tugrott erre a cmre
start:

	; --- STACK belltsa a vgre --- --- --- --- --- --- --- ---
	ldi	tmp, HIGH(RAMEND)	; stack bellts
	out	SPH, tmp		; a memria vgre teszi
	ldi	tmp, LOW(RAMEND)	; a stack pointer-eket
	out 	SPL, tmp

	; --- --- --- --- --- --- --- --- --- --- ---
	ldi tmp, 0b_1000_0000	; 3 szn LED RED lba itt (0x80 = 0b_1000_0000) 
	out DDRC, tmp		; PORTC legfels bit kimenetre lltsa 

	ldi tmp, 0b_0000_1100	; 3 szn LED GREEN s BLUE lba itt (0x0C = 0b_0000_1100)
	out DDRE, tmp		; PORTE megfelel lbainak kimenetre lltsa 

ciklus:	
	ldi	tmp, 0b_1000_0000	; RED meghajtsa, egyszerre csak egy 2 soros rsz legyen aktv
	out PORTC, tmp

;	ldi	tmp, 0b_0000_1000	; GREEN meghajtsa, egyszerre csak egy 2 soros rsz legyen aktv
;	out PORTE, tmp

;	ldi	tmp, 0b_0000_0100	; BLUE meghajtsa, egyszerre csak egy 2 soros rsz legyen aktv
;	out PORTE, tmp

	jmp ciklus

vege:					; BAB
	jmp vege

;==============================================================================
; 8. FELADAT: STACK store and load (PLUSSZ, OPCIONLIS PLDA, nem rai anyag)
;------------------------------------------------------------------------------
;
;	- az SREG rtkelnek figyelse elmentse, visszarsa plda (HA KELL)
;
;==============================================================================

.include "m128def.inc"
.def tmp =r16	
.def reg =r17

.org 0x0		
	rjmp start

.org 0x100	
start:

	ldi tmp, high(RAMEND)
	out sph, tmp
	ldi tmp, low (RAMEND)
	out spl, tmp

	clr tmp

	in reg, sreg	; 'sreg' rtknek elmentse egy 'reg' nev regiszterbe
	push reg	; 'reg' regiszter elmentse a STACK-be

	clr reg
	inc tmp

	pop reg		; 'reg' regiszterbe STACK-bl visszaolvass
	out sreg, reg	; 'reg' regiszterbl a 'sreg'-be visszarom a rgi 'sreg' rtkeket

vege:	
	jmp vege

